/*  nifticonverter.js: script to manage the user interface nifticonverter.ui and to communicate with the NIfTI converter plugin.
    This bundle consists of *.js, *.ui and *.dylib/*.dll/*.so files, place in /Documents/BVQXExtensions/Plugins_32 or 64 (Mac)/ folder.
    Functionality: Converts between NIfTI and BrainVoyager file formats. 
    Usage: In the BrainVoyager menu, select 'Plugins' > NIfTI converter
    Hester Breman, 2003-2016;
    Latest update: 03-05-13: in prepareConversion() and read3FilesTextFile(): added line to also look at target extension for opening VMR
    01-02-2016 (fixed zeropadding for VMRs)
    28-02-2013 (removed printouts)
*/

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DECLARATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "NIfTI converter"; 
scriptObj.dialogFileName 		= "nifticonverter_v110.ui"; 
scriptObj.dialogAccessName      = "niftiDlg"; 

// global variables
var command = "Unknown";
var incompleteinfo = false;
var hasposfile = false;
var hasniftifile = false;
var hastrffile = false;
var openInBVQX = false;
var doBatch = false;

// file type IDs correspond to plugin converter's IDs
const FILE_TYPE_UNKNOWN = 0;
const NIFTI_FILE_TYPE = 1; 
const NIFTI_PAIR_FILE_TYPE = 2;
const ANALYZE75_FILE_TYPE = 3;
const NIFTI_IMG_TYPE = 4;
const BV_TYPE_VMR = 20;
const BV_TYPE_FMR = 21;
const BV_TYPE_AMR = 22;
const BV_TYPE_VTC = 24;
const BV_TYPE_VOI = 27;
const BV_TYPE_DMR = 29;
const BV_TYPE_VMP = 201;
const BV_TYPE_VMP_AR = 202;
const BV_TYPE_VMP_NR = 203;
const FILE_TYPE_TXT = 401;
const FILE_TYPE_TXT_BATCH = 402;
const FILE_TYPE_TXT_PARAMS = 403;
const BV_TYPE_MAP = 200;
const BV_TYPE_STC = 23;
const BV_TYPE_DWI = 30;
const BV_TYPE_VDW = 31;
const BV_TYPE_GLM = 205;
const BV_TYPE_AVA = 206;
const BV_TYPE_PRT = 208;
const BV_TYPE_TRF = 301;
const BV_TYPE_POS = 302;
// submenu options
const PRINT_TO_LOG = 1;
const PRINT_TO_FILE = 2;
const PRINT_TO_LOG_AND_FILE = 3;
const PRINT_HIST = 4;
// 	enum ConvMenuOptions
const CONVERT_SINGLE_FILE = 1;
const CONVERT_BATCH = 2;
const CONVERT_BATCH_W_PARAMS = 3;
//const CONVERT_REGISTERED = 3;
//const CONVERT_REGISTERED_BATCH = 4;
// enum POSMENUOPTIONS
const POS_MATRIX_FROM_FILE = 1;
const POS_ADD_TO_FILE = 2;
const POS_BVQXPOS_FROM_NIIPOS = 3; // not active yet
const POS_COMPUTE_COORDS_OF_POINTS = 4; // not active yet
// enum TRFMENUOPTIONS
const TRF_INVERT_MAT = 1;
const TRF_APPLY_TRF_MAN = 2;
const TRF_APPLY_TRF_FILE = 3;
const TRF_COMPUTE_BBOX = 4;

// ~~~ nicknames for children ~~~
// global tab
var optConvertButton, optInfoButton, optPosButton, optManipButton, optTrfButton, optBatchButton, startButton;
var niftiNameLabel, getNiftiFileButton;
var btnGetParamsFile, lblParamsFile, lblBatchFile, getBatchConvButton;
// conversion tab
var niiFromButton, hdrFromButton, vmrFromButton, fmrFromButton, dmrFromButton, vtcFromButton, vmpFromButton, voiFromButton; 
var niiToButton, hdrToButton, vmrToButton, fmrToButton, dmrToButton, vtcToButton, vmpToButton; 
var transformBox, getNIfTIConvButton, transformBox, flipBox, batchBox, batchFileLabel, openInBVQXBox, zeropadBox, intensityBox, vointerpBox, resetBox;
var chbSelectVols, spbFromVol, spbToVol;
// info tab
var niiPrintButton, hdrPrintButton, vmrPrintButton, fmrPrintButton, dmrPrintButton, vtcPrintButton, vmpPrintButton, voiPrintButton; 
var getniftiInfoButton, destBothButton, destFileButton, destLogButton; 
// positioning tab
var getPosFileButton, createPosFileButton, posToImgButton, printPosMatButton, calcPosButton, p00, p01, p02, p03, p10, p11, p12, p13, p20, p21, p22, p23, p30, p31, p32, p33;
var nxBox, nyBox, nzBox, dxBox, dyBox, dzBox;
// header manipulation tab
var optCopyButton, optFldManipButton; 
var leQformCode, leQb, leQc, leQd, leQoffsetX, leQoffsetY, leQoffsetZ, leSformCode, leSrow00, leSrow01, leSrow02,leSrow03, leSrow11 ,leSrow12,leSrow13, leSrow21, leSrow22, leSrow23;
var rbtnCopyPos,rbtnCopyAll, rbtnNiiType, rbtnHdrType, rbtnNiiTarget, rbtnHdrTarget, lblOtherNIfTIfile, btnOtherNIfTIfile;
var leQFac, lblNx, lblNy, lblNz, lblDx, lblDy, lblDz, calcBVposButton;
// transformation tab
var getNIfTIFileToTrfButton, invertTrfButton,applyManTrfButton,applyBVQXTrfButton, getTrfFileTypeBox, transformGrBox, t00, t01, t02, t03, t10, t11, t12, t13, t20, t21, t22, t23, t30, t31, t32, t33;
var comboboxListenerButton; // hidden button to listen to filetype changes 
 // batch tab
var writeBatchFileButton;



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ GENERIC FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  



scriptObj.errorfree = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr == 0) return true;
    else return false;
}

scriptObj.checkerrorstatus = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
            BrainVoyagerQX.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.start = function() {
    
    var success = true;   
    var bvqxdocarray; 
    // check if status is prepared 
    if (optConvertButton.checked == 1) {
        this.prepareConversion();
    } else if (optInfoButton.checked == 1) {   
        success = this.prepareCollectInfo();
    } else if (optBatchButton.checked == 1) { 
        bvqxdocarray = this.prepareBatchFuncs(); // if the files are BrainVoyager project files, they will be opened now.
    } else {
        BrainVoyagerQX.PrintToLog("Could not find which option was checked.");
        incompleteinfo = true;
    }   
    
    // then start plugin
    if ((!incompleteinfo) && (success)) {
        BrainVoyagerQX.PrintToLog("Send start signal to plugin...");
        BrainVoyagerQX.SetPluginStringParameter("Command", "Start");    
        BrainVoyagerQX.ExecutePlugin(); 
    
        // ask parameters from plugin
        if ((this.errorfree()) && (!doBatch) && (openInBVQX)) {
            var convertedname = BrainVoyagerQX.GetPluginStringParameter("ConvertedName");
            BrainVoyagerQX.PrintToLog("Opening " + convertedname + "...");
            var bvqxdoc = BrainVoyagerQX.OpenDocument(convertedname.toString());
        }
    } else {
        BrainVoyagerQX.PrintToLog("Cannot start function: incomplete information. Please check whether all parameters have been collected.");
    }   
    
    // now close the dialog
    this.niftiDlg.close(); 
}

scriptObj.getNiftiFile = function() {

    var filetypeid = FILE_TYPE_UNKNOWN;
    if (optConvertButton.checked == 1) {
        filetypeid = this.getConvertFromFileId();
    } else if (optInfoButton.checked == 1) {
        filetypeid = this.getInfoFileId();
    } else {
        BrainVoyagerQX.PrintToLog("Source of signal to get NIfTI file not recognised.");
    }
                  
    if (filetypeid != FILE_TYPE_UNKNOWN) {          
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");    
        BrainVoyagerQX.SetPluginIntParameter("FileID", filetypeid);
        BrainVoyagerQX.ExecutePlugin(); 
        var niftifilename = BrainVoyagerQX.GetPluginStringParameter("NiftiFileName");
        niftiNameLabel.text = niftifilename;
        hasniftifile = true;  
    }    
} 
 
scriptObj.isniftifile = function(filetypeid) {
    
    if ((filetypeid == NIFTI_FILE_TYPE) || (filetypeid == NIFTI_PAIR_FILE_TYPE) || (filetypeid == ANALYZE75_FILE_TYPE) || 
        (filetypeid == NIFTI_IMG_TYPE)) return true;
    else return false;
}


scriptObj.printMatrixToLog = function(matrix) {

    BrainVoyagerQX.PrintToLog("Matrix:");
    for (i=0; i<4; i++) {
            BrainVoyagerQX.PrintToLog(matrix[i][0] + ", " + matrix[i][1] + ", " + matrix[i][2] + ", " + matrix[i][3]); 
    }
}

scriptObj.onlyHasNumericValues = function(matrix) {

    var value;
    var hasOnlyNumericValues = true;
    for (i=0; i<4; i++) {
        for (j=0; j<4; j++) {
           if (isNaN(matrix[i][j])) {
                hasOnlyNumericValues = false;
                break;
            }
        }            
    }
    return hasOnlyNumericValues;
}

scriptObj.setMatrixParametersForPlugin = function(matrix) {

    BrainVoyagerQX.SetPluginFloatParameter("m00", matrix[0][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m01", matrix[0][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m02", matrix[0][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m03", matrix[0][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m10", matrix[1][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m11", matrix[1][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m12", matrix[1][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m13", matrix[1][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m20", matrix[2][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m21", matrix[2][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m22", matrix[2][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m23", matrix[2][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m30", matrix[3][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m31", matrix[3][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m32", matrix[3][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m33", matrix[3][3]);
}


scriptObj.getMatrixParametersFromPlugin = function() {

    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = BrainVoyagerQX.GetPluginFloatParameter("m00");
    matrix[0][1] = BrainVoyagerQX.GetPluginFloatParameter("m01");
    matrix[0][2] = BrainVoyagerQX.GetPluginFloatParameter("m02");
    matrix[0][3] = BrainVoyagerQX.GetPluginFloatParameter("m03");
             
    matrix[1][0] = BrainVoyagerQX.GetPluginFloatParameter("m10");
    matrix[1][1] = BrainVoyagerQX.GetPluginFloatParameter("m11");
    matrix[1][2] = BrainVoyagerQX.GetPluginFloatParameter("m12");
    matrix[1][3] = BrainVoyagerQX.GetPluginFloatParameter("m13");
    
    matrix[2][0] = BrainVoyagerQX.GetPluginFloatParameter("m20");
    matrix[2][1] = BrainVoyagerQX.GetPluginFloatParameter("m21");
    matrix[2][2] = BrainVoyagerQX.GetPluginFloatParameter("m22");
    matrix[2][3] = BrainVoyagerQX.GetPluginFloatParameter("m23");
    
    matrix[3][0] = BrainVoyagerQX.GetPluginFloatParameter("m30");
    matrix[3][1] = BrainVoyagerQX.GetPluginFloatParameter("m31");
    matrix[3][2] = BrainVoyagerQX.GetPluginFloatParameter("m32");
    matrix[3][3] = BrainVoyagerQX.GetPluginFloatParameter("m33");        
    
    return matrix;
}

scriptObj.getIdentityMatrix = function() {

    var matrix = [[1.0,0,0,0],[0,1.0,0,0],[0,0,1.0,0],[0,0,0,1.0]]; 
    return matrix; 
}

scriptObj.getEmptyMatrix = function() {

    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    return matrix; 
}

scriptObj.multiply4x4Matrices = function(mat1, mat2) {

        var mat3 = this.getEmptyMatrix();
		mat3[0][0] = mat1[0][0]*mat2[0][0]+ mat1[0][1]*mat2[1][0]+ mat1[0][2]*mat2[2][0]+ mat1[0][3]*mat2[3][0];
		mat3[0][1] = mat1[0][0]*mat2[0][1]+ mat1[0][1]*mat2[1][1]+ mat1[0][2]*mat2[2][1]+ mat1[0][3]*mat2[3][1];
		mat3[0][2] = mat1[0][0]*mat2[0][2]+ mat1[0][1]*mat2[1][2]+ mat1[0][2]*mat2[2][2]+ mat1[0][3]*mat2[3][2];
		mat3[0][3] = mat1[0][0]*mat2[0][3]+ mat1[0][1]*mat2[1][3]+ mat1[0][2]*mat2[2][3]+ mat1[0][3]*mat2[3][3];
        
		mat3[1][0] = mat1[1][0]*mat2[0][0]+ mat1[1][1]*mat2[1][0]+ mat1[1][2]*mat2[2][0]+ mat1[1][3]*mat2[3][0];
		mat3[1][1] = mat1[1][0]*mat2[0][1]+ mat1[1][1]*mat2[1][1]+ mat1[1][2]*mat2[2][1]+ mat1[1][3]*mat2[3][1];
		mat3[1][2] = mat1[1][0]*mat2[0][2]+ mat1[1][1]*mat2[1][2]+ mat1[1][2]*mat2[2][2]+ mat1[1][3]*mat2[3][2];
		mat3[1][3] = mat1[1][0]*mat2[0][3]+ mat1[1][1]*mat2[1][3]+ mat1[1][2]*mat2[2][3]+ mat1[1][3]*mat2[3][3];
		
        mat3[2][0] = mat1[2][0]*mat2[0][0]+ mat1[2][1]*mat2[1][0]+ mat1[2][2]*mat2[2][0]+ mat1[2][3]*mat2[3][0];
		mat3[2][1] = mat1[2][0]*mat2[0][1]+ mat1[2][1]*mat2[1][1]+ mat1[2][2]*mat2[2][1]+ mat1[2][3]*mat2[3][1];
		mat3[2][2] = mat1[2][0]*mat2[0][2]+ mat1[2][1]*mat2[1][2]+ mat1[2][2]*mat2[2][2]+ mat1[2][3]*mat2[3][2];
		mat3[2][3] = mat1[2][0]*mat2[0][3]+ mat1[2][1]*mat2[1][3]+ mat1[2][2]*mat2[2][3]+ mat1[2][3]*mat2[3][3];
		
        mat3[3][0] = mat1[3][0]*mat2[0][0]+ mat1[3][1]*mat2[1][0]+ mat1[3][2]*mat2[2][0]+ mat1[3][3]*mat2[3][0];
		mat3[3][1] = mat1[3][0]*mat2[0][1]+ mat1[3][1]*mat2[1][1]+ mat1[3][2]*mat2[2][1]+ mat1[3][3]*mat2[3][1];
		mat3[3][2] = mat1[3][0]*mat2[0][2]+ mat1[3][1]*mat2[1][2]+ mat1[3][2]*mat2[2][2]+ mat1[3][3]*mat2[3][2];
		mat3[3][3] = mat1[3][0]*mat2[0][3]+ mat1[3][1]*mat2[1][3]+ mat1[3][2]*mat2[2][3]+ mat1[3][3]*mat2[3][3];
        return mat3;
}

function getdeterminant(a){
// Get determinant of 4 x 4 matrix
// courtesy of John A.
  var dt = mat[0][0]*( mat[1][1]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][1]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[3][1]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2]))
        +mat[1][0]*( mat[0][1]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][1]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][1]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2]))
        +mat[2][0]*( mat[0][1]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[1][1]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][1]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]))
        +mat[3][0]*( mat[0][1]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2])
		+mat[1][1]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2])
		+mat[2][1]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
		return(dt);
}

function getinverse(mat){
// get inverse of 4 x 4 matrix
// courtesy of John A.

	var idet    = 1.0/getdeterminant(mat);
    var invmat = getIdentityMatrix();

    invmat[0][0] = idet*(mat[1][1]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][1]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[3][1]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2]));
    invmat[0][1] = idet*(mat[0][1]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][1]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][1]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2]));
    invmat[0][2] = idet*(mat[0][1]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[1][1]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][1]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]));
    invmat[0][3] = idet*(mat[0][1]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2])
		+mat[1][1]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2])
		+mat[2][1]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
    invmat[1][0] = idet*(mat[1][0]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][0]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[3][0]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2]));
    invmat[1][1] = idet*(mat[0][0]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][0]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][0]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2]));
    invmat[1][2] = idet*(mat[0][0]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[1][0]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][0]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
    invmat[1][3] = idet*(mat[0][0]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2])
		+mat[1][0]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2])
		+mat[2][0]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]));
    invmat[2][0] = idet*(mat[1][0]*( mat[2][1]*mat[3][3]-mat[2][3]*mat[3][1])
		+mat[2][0]*(-mat[1][1]*mat[3][3]+mat[1][3]*mat[3][1])
		+mat[3][0]*( mat[1][1]*mat[2][3]-mat[1][3]*mat[2][1]));
    invmat[2][1] = idet*(mat[0][0]*(-mat[2][1]*mat[3][3]+mat[2][3]*mat[3][1])
		+mat[2][0]*( mat[0][1]*mat[3][3]-mat[0][3]*mat[3][1])
		+mat[3][0]*(-mat[0][1]*mat[2][3]+mat[0][3]*mat[2][1]));
    invmat[2][2] = idet*(mat[0][0]*( mat[1][1]*mat[3][3]-mat[1][3]*mat[3][1])
		+mat[1][0]*(-mat[0][1]*mat[3][3]+mat[0][3]*mat[3][1])
		+mat[3][0]*( mat[0][1]*mat[1][3]-mat[0][3]*mat[1][1]));
    invmat[2][3] = idet*(mat[0][0]*(-mat[1][1]*mat[2][3]+mat[1][3]*mat[2][1])
		+mat[1][0]*( mat[0][1]*mat[2][3]-mat[0][3]*mat[2][1])
		+mat[2][0]*(-mat[0][1]*mat[1][3]+mat[0][3]*mat[1][1]));
    invmat[3][0] = idet*(mat[1][0]*(-mat[2][1]*mat[3][2]+mat[2][2]*mat[3][1])
		+mat[2][0]*( mat[1][1]*mat[3][2]-mat[1][2]*mat[3][1])
		+mat[3][0]*(-mat[1][1]*mat[2][2]+mat[1][2]*mat[2][1]));
    invmat[3][1] = idet*(mat[0][0]*( mat[2][1]*mat[3][2]-mat[2][2]*mat[3][1])
		+mat[2][0]*(-mat[0][1]*mat[3][2]+mat[0][2]*mat[3][1])
		+mat[3][0]*( mat[0][1]*mat[2][2]-mat[0][2]*mat[2][1]));
    invmat[3][2] = idet*(mat[0][0]*(-mat[1][1]*mat[3][2]+mat[1][2]*mat[3][1])
		+mat[1][0]*( mat[0][1]*mat[3][2]-mat[0][2]*mat[3][1])
		+mat[3][0]*(-mat[0][1]*mat[1][2]+mat[0][2]*mat[1][1]));
    invmat[3][3] = idet*(mat[0][0]*( mat[1][1]*mat[2][2]-mat[1][2]*mat[2][1])
		+mat[1][0]*(-mat[0][1]*mat[2][2]+mat[0][2]*mat[2][1])
		+mat[2][0]*( mat[0][1]*mat[1][2]-mat[0][2]*mat[1][1]));
        
    return invmat;
}

scriptObj.getPath = function(filename) {
    
    var start = 0; // start searching from begin 
    var last = filename.lastIndexOf("/"); // search for last path separator 
    var path = filename.substring(start, (last+1)); // now return path without filename 
    return path; 
}

scriptObj.removeExtension = function(filename) {

    var start = 0; // start searching from begin 
    var posExt = filename.lastIndexOf("."); // search for `extension separator' 
    var nameWithoutExt = filename.substring(start, posExt); // get name without extension 
    return nameWithoutExt;
}

scriptObj.getExtension = function(filename) {

    var start = 0; // start searching from begin 
    var posExt = filename.lastIndexOf("."); // search for `extension separator' 
    var ext = filename.substring(posExt, filename.length); // get extension without name, including '.'
    return ext;
}

scriptObj.getFileExtension = function(filename) {

    var posOfDot = filename.lastIndexOf(".");
    var extension = filename.substring((posOfDot+1), filename.length);
    return extension;
}

// exchange extension of filename; new extension should contain '.', like ".txt"
scriptObj.exchangeExtension = function(filename, newExt) {

    var newName = this.removeExtension(filename) + newExt;
    return newName;
}

// add a addition to filename: <filename>_addition.<ext>    
scriptObj.changeFileName = function(filename, addition) {

    var nameWithoutExt = this.removeExtension(filename);
    var newName = nameWithoutExt + "_" + addition + this.getExtension(filename);
    return newName;
}    


scriptObj.readTextFile = function(name, printtolog) {
/* read a file with the following structure:
<nroffiles>
<filename 1>
<target file extension 1>
 ...
<filename n>
<target file extension n>

returns an n x 2 array
*/
 
    var filecounter; 
    BrainVoyagerQX.PrintToLog("Reading: " + name);
    var filenamesfile = new QFile(name); 
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly)); 
    var textstr = new QTextStream(filenamesfile); 
    var nroffiles = parseInt(textstr.readLine()); 
    if (printtolog) {
        BrainVoyagerQX.PrintToLog("Number of files in text file: " + nroffiles);
    }
    var projfilenames = new Array(nroffiles);
    for (filecounter = 0; filecounter < nroffiles; filecounter++) {
        var filename = textstr.readLine(); 
        if (printtolog) BrainVoyagerQX.PrintToLog("Filename: " + filename);
        var targetfiletype = textstr.readLine();
        var filepair = new Array(filename, targetfiletype);
        BrainVoyagerQX.PrintToLog(filename); 
        projfilenames[filecounter] = filepair;
    } 
    filenamesfile.close(); 
    return projfilenames;
}

scriptObj.read3FilesTextFile = function(name, printtolog) {
/* read a file with the following structure:
<nroffiles>
<filename 1>
<target file extension 1>
<filename 2> (extension: vtc, vmp, voi)
<target file extension 2>
<vmr basis file>
 ...
<filename n>
<target file extension n>

returns an n x 2 array
*/
 
    var filecounter; 
    BrainVoyagerQX.PrintToLog("Reading: " + name);
    var filenamesfile = new QFile(name); 
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly)); 
    var textstr = new QTextStream(filenamesfile); 
    var nroffiles = parseInt(textstr.readLine()); 
    if (printtolog) {
        BrainVoyagerQX.PrintToLog("Number of files in text file: " + nroffiles);
    }
    var projfilenames = new Array(nroffiles);
    for (filecounter = 0; filecounter < nroffiles; filecounter++) {
        var filename = textstr.readLine(); 
        if (printtolog) BrainVoyagerQX.PrintToLog("Filename: " + filename);
        var targetfiletype = textstr.readLine();
        var anatname = "";
     //   if (((filename.indexOf("vtc") > 0) || (filename.indexOf("vdw") > 0) || (filename.indexOf("vmp") > 0) || (filename.indexOf("voi") > 0)) && (targetfiletype.indexOf("nii") > 0)) {
        if ((filename.indexOf(".vtc") > 0) || (filename.indexOf(".vmp") > 0) || (filename.indexOf(".voi") > 0) ||
            (targetfiletype.indexOf("vmp") > -1) || (targetfiletype.indexOf("vtc") > -1)|| (targetfiletype.indexOf("voi") > -1)) { // 03-05-13: added vmp, vtc, voi
           anatname = textstr.readLine();           
        }
        var filecluster = new Array(filename, targetfiletype, anatname);
        projfilenames[filecounter] = filecluster;
    } 
    filenamesfile.close(); 
    return projfilenames;
}


scriptObj.CloseAllProjects = function() {

	var docs = BrainVoyagerQX.Documents;
	var n_docs = docs.Count;
	var doc;
	while (n_docs != 0) {
		n_docs = docs.Count;
		if (n_docs > 0) {
			doc = BrainVoyagerQX.ActiveDocument;
			doc.Close();
		}
	}
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 1. CONVERSIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareConversion = function() {

    BrainVoyagerQX.PrintToLog("Prepare conversion...");  
    doBatch = true;
    
    // set parameters for plugin // const CONVERT_REGISTERED = 3; // const CONVERT_REGISTERED_BATCH = 4;
  //  if ((fromID != FILE_TYPE_UNKNOWN) && (toID != FILE_TYPE_UNKNOWN)) { 
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "Convert"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", CONVERT_BATCH_W_PARAMS);
        BrainVoyagerQX.SetPluginIntParameter("BatchFileName", batchFileLabel.text); 
        BrainVoyagerQX.SetPluginIntParameter("ParamFileName", lblParamsFile.text);
    
    // first close all documents
    BrainVoyagerQX.PrintToLog("Closing all other documents...");
   var closedDocs = this.CloseAllProjects();
   // if the files are BrainVoyager projects, then they should be opened
   var filename = String(batchFileLabel.text); // note: can we assume that there is a filename here? 
   if (filename.length > 0) {
    BrainVoyagerQX.PrintToLog("Reading text file: " + filename);
    var projfls = this.read3FilesTextFile(filename, true); //this.readTextFile(filename, true); // true: print to log    
    var bvqxdocarray = new Array(); 
    // 
    var filenr;
    for (filenr = 0; filenr < projfls.length; filenr++) {
        var filename = String(projfls[filenr][0]);
        var targetext = String(projfls[filenr][1]);// String: 03-05-13        
        if ((filename.indexOf(".vmr") > 0) || (filename.indexOf(".dmr") > 0) || (filename.indexOf(".fmr") > 0)) {
            var doc = BrainVoyagerQX.OpenDocument(filename);
            bvqxdocarray.push(doc);                    
        } else if ( (filename.indexOf(".vtc") > 0) || (filename.indexOf(".vmp") > 0) || (filename.indexOf(".voi") > 0) || 
                    (targetext.indexOf("vmp") > -1) || (targetext.indexOf("vtc") > -1)|| (targetext.indexOf("voi") > -1)) {// 03-05-13
            var anatname = String(projfls[filenr][2]);
            var basisvmr = BrainVoyagerQX.OpenDocument(anatname);
            bvqxdocarray.push(basisvmr); 
        } else {
            BrainVoyagerQX.PrintToLog("File extension not recognised. Please check results carefully.");
        }
        // end if
    } // end for
   } else { 
        BrainVoyagerQX.PrintToLog("No text file name available.");
   }
   return bvqxdocarray;

}
 
scriptObj.getBatchFile = function() {
 
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");
    BrainVoyagerQX.SetPluginIntParameter("FileID", FILE_TYPE_TXT); 
    BrainVoyagerQX.SetPluginIntParameter("TextFileType", FILE_TYPE_TXT_BATCH);     
    BrainVoyagerQX.ExecutePlugin();  
    var batchfilename = BrainVoyagerQX.GetPluginStringParameter("BatchFileName");
    batchFileLabel.text = batchfilename;
} 

scriptObj.getParamsFile = function() {

    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");
    BrainVoyagerQX.SetPluginIntParameter("FileID", FILE_TYPE_TXT);
    BrainVoyagerQX.SetPluginIntParameter("TextFileType", FILE_TYPE_TXT_PARAMS); 
    BrainVoyagerQX.ExecutePlugin(); 
    var paramfilename = BrainVoyagerQX.GetPluginStringParameter("ParamFileName");
    lblParamsFile.text = paramfilename;
}

scriptObj.enableBatchButton = function() {

    var doBatch = batchBox.checked;
    if (doBatch) getBatchConvButton.enabled = true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 2. INFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareCollectInfo = function() {

    // get information from widgets
    var success = true;
    var submenuoption;
    var fileid = this.getInfoFileId();
    if (destLogButton.checked == 1) {
        submenuoption = PRINT_TO_LOG;
    } else if (destFileButton.checked == 1) {
        submenuoption = PRINT_TO_FILE;
    } else if (destBothButton.checked == 1) {
        submenuoption = PRINT_TO_LOG_AND_FILE;
    } else {
        incompleteinfo = true;
        success = false;
    }
  
    // set parameters for plugin 
    if ((fileid != FILE_TYPE_UNKNOWN) && (!incompleteinfo)) {
    
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "PrintInfo"); 
        BrainVoyagerQX.SetPluginIntParameter("FileID", fileid);
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
        if (this.isniftifile(fileid)) {
            if (hasniftifile) {        
                BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
            } else {
                incompleteinfo = true;
                success = false;
            }
        }
    } else {
        success = false;
    }
    return success;
}

scriptObj.getInfoFileId = function() {

    var fileid;
    if (niiPrintButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrPrintButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrPrintButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrPrintButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrPrintButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcPrintButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpPrintButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else if (voiPrintButton.checked == 1) {
       fileid = BV_TYPE_VOI;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 3. BATCH FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareBatchFuncs = function() {

    // set parameters for plugin
   BrainVoyagerQX.SetPluginStringParameter("MenuOption", "BatchFuncs"); 
   
 }

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MAIN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 

    this.niftiDlg.windowTitle = "NIfTI import (batch version)"; 

    // ~~~ make nicknames for children ~~~
	var bvqx = BrainVoyagerQX;
    startButton = bvqx.FindChild("btnStart"); 
    
    // global tab
    optConvertButton    = bvqx.FindChild("rbtnBatchConvert");
    optInfoButton       = bvqx.FindChild("rbtnImgInfo");
    optBatchButton      = bvqx.FindChild("rbtnBatchFunctions");
    getNiftiFileButton  = bvqx.FindChild("btnGetNIfTIFile");
    niftiNameLabel      = bvqx.FindChild("lblNIfTIname");
    
    // 1. convert
    getBatchConvButton  = bvqx.FindChild("btnBatchFile");
    batchFileLabel      = bvqx.FindChild("lblBatchFile");
    lblParamsFile       = bvqx.FindChild("lblParamsFile");
    btnGetParamsFile    = bvqx.FindChild("btnGetParamsFile");

     // 2. get information
    niiPrintButton = bvqx.FindChild("rbtnPrintNII"); 
    hdrPrintButton = bvqx.FindChild("rbtnPrintHDR");
    vmrPrintButton = bvqx.FindChild("rbtnPrintVMR"); 
    fmrPrintButton = bvqx.FindChild("rbtnPrintFMR"); 
    dmrPrintButton = bvqx.FindChild("rbtnPrintDMR"); 
    vtcPrintButton = bvqx.FindChild("rbtnPrintVTC"); 
    vmpPrintButton = bvqx.FindChild("rbtnPrintVMP");
    voiPrintButton = bvqx.FindChild("rbtnPrintVOI");
    destBothButton = bvqx.FindChild("rbtnDestBoth");
    destFileButton = bvqx.FindChild("rbtnDestFile"); 
    destLogButton = bvqx.FindChild("rbtnDestLog"); 
    
    // 3. batch facilitation
    writeBatchFileButton = bvqx.FindChild("rbtnWriteBatch");

    // start
    startButton.clicked.connect(this, this.start);

    // connect widgets to functions
    getBatchConvButton.clicked.connect(this, this.getBatchFile);    
    getNiftiFileButton.clicked.connect(this, this.getNiftiFile);
    btnGetParamsFile.clicked.connect(this, this.getParamsFile);
    
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
